/* Definition des Videodaten-Formats
 * PKS, SS06, IBR, ZK, kurtisi@ibr.cs.tu-bs.de
 */

#ifndef AVS_H
#define AVS_H

#include <sys/types.h>

/* hier liegen die Videos */
#define MOVIE_DIRECTORY "/home/kurtisi/pks_public/AudioVideo/movies/"

/* beim Client sind maximal 80 Frames vorzuhalten */
#define MAX_FRAME_BUF	80

/* Marker fuer die verschiedenen Header-Typen */
#define MARKER_MOVIE	0x003071e0
#define MARKER_FRAME	0x00f8a3e0
#define MARKER_VIDEO	0x0071de00
#define MARKER_AUDIO	0x00a0d100


/* Jede .ibrv-Datei beginnt mit Film-Header */
typedef struct
{
	int marker;				// MARKER_MOVIE
	int width;				// Aufloesung der Videos 
	int height;				// (hier konstant: 320*240)
	int frames;				// Anzahl enthaltener Frames
	int bits_per_pixel;		// hier 24 bpp
	int bytes_per_frame;	// Groess eines Bildes in Bytes
	int video_block_num;	// Anzahl der Video-Bloecke pro Bild
	int video_block_size;	// Groesse der unkomprimierten Video-Bloecke
	int audio_block_size;	// Groesse der Audiodaten
} movie_header;

/* Es folgen [frames] Frames, jeweils beginnend mit einem Frame-Header */
typedef struct
{
	int marker;				// MARKER_FRAME
	int frame_num;			// Frame-Index
	int time_ms;			// Zeit seit Beginn des Films (hier konstante fps von 12.5)
	int video_block_count;	// Anzahl enthaltener Video-Bloecke (hier 6)
} frame_header;

/* Es folgen [video_block_count] Video-Bloecke, jeweils beginnend mit */
typedef struct
{
	int marker;				// MARKER_VIDEO
	int frame_num;			// Frame-Index
	int video_block_num;	// Video-Block-Index
	int video_block_size;	// Laenge des komprimierten Video-Blocks
} video_header;

/* Und am Ende des Frames das Audio mit */
typedef struct
{
	int marker;				// MARKER_AUDIO
	int frame_num;			// Frame-Index
	int audio_block_size;	// Laenge des Audio Blocks, hier 640 (8kHz bei 12.5 fps)
} audio_header;


/* im Geruest bereitgestellte Funktionen 
 * zur Audio- und Video-Ausgabe          
 ***************************************/

/* initialisiere Audio und Video
 *   - initialisiert Audio-Ausgabe ueber PortAudio
 *   - initialisiert Video-Ausgabe ueber SDL
 *   - legt Buffer fuer die Dekomprimierung der JPEG-Bildsegmente an
 *   - movie_header enthaelt Video-Parameter
 *   - picnum ist Anzahl der zu buffernden dekomprimierten Bilder (< MAX_FAME_BUF)
 */
void init_audio_video(movie_header *video, int picnum);

/* decodiere JPEG-Segment
*   - offset ist absolut
*/
int decomp_JPEG (char* data, int data_size, int offset);

/* zeige das Bild bei y_offset an
 *   - y_offset ist erste Zeilennummer des darzustellenden Bildes
 */
void show_video(int y_offset);

/* schreibe einen Audio-Block in den Audio-Eingangspuffer
 *   - die in 8bit-mulaw vorliegenden Audiodaten werden vor der Ausgabe konvertiert
 */
int play_audio_block(char *data, size_t length);

/* gib die Anzahl der in das Audio-Device geschriebenen Audio-Bloecke zurueck
 *   - im Audio-Device selbst werden 3 Bloecke gepuffert
 *   - eignet sich zur Synchronisierung auf Audio
 */
int get_completed_audio_blocks();

#endif
